#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include "glaux.h"

#include "CVect3D.hpp"

#ifndef NULL
#define NULL 0x00
#endif

#ifndef M_PI
#define M_PI		3.14159265358979323846 
#endif

#define WINDOW_WIDTH  800
#define WINDOW_HEIGHT 600
#define WINDOW_TITLE  "Horska draha by Jan Skalicky"

#define MOTION_TIMER 20
/* if 1 meter ~ 1.0 in scene, speed of motion will be 8 m/s */
#define MOTION_STEP  (8.0 / MOTION_TIMER)
#define MOTION_ANGLE (2.0 / MOTION_TIMER)
#define MOTION_MOUSE (400.0 * MOTION_STEP)
#define SUN_VELOCI 0.0025

#define SURFACE_XMAX 64
#define SURFACE_ZMAX 64
/* horizontal gap between 2 near square corner points */
#define SURFACE_GAP 2.5f
/* minimal vertical difference between corner and nearby inner point */
#define SURFACE_UP2 1.0f
/* maximal level of top/bottom point in surface (one level is corner-middle, so between 2 corners can be 0 or 2 level diff */
#define SURFACE_TOP 100

/* random number from 0 to n */
#define RAND0N(n) ( (int)(rand() / ( ((unsigned)RAND_MAX+1) / ((n)+1) )) )

using namespace std;

void guiDisplay();
void guiReshape( int, int );
void guiIdle();
void guiTimer( int );
void guiKeyboard( unsigned char, int, int );
void guiKeyboardUp( unsigned char, int, int );
void guiMouse( int, int, int, int );
void guiMouseActive( int, int );
void guiMousePassive( int, int );

int generateSurface();
void generateSurfaceLift( unsigned int, unsigned int, int, char (*)[SURFACE_ZMAX] );
void drawSurface();
void drawTree( float );
void drawSkybox();
int NactiTexturu (char *filename, unsigned int* handle);
void ZrusTexturu (unsigned int* handle);

unsigned char keys[256];
unsigned char fog = 0;
unsigned char normals = 0;
unsigned char light = 0;
unsigned char wired = 0;
unsigned char torch = 0;
unsigned char textures = 0;
unsigned char tea_motion = 1;
unsigned char tea_attach = 1;
double        tea_veloci = 0.01; /* angle velocity of teapot */
CVect3D sun(0.0, 1.0, -0.2);
CVect3D tea_pos( SURFACE_XMAX*SURFACE_GAP/2, 0.0, -SURFACE_ZMAX*SURFACE_GAP/4 );

/* surface outer points (4 in corners) heights */
int surface[SURFACE_XMAX][SURFACE_ZMAX];
/* surface inner points (1 in the middle) heights */
int surface_mids[SURFACE_XMAX-1][SURFACE_ZMAX-1];
/* surface outer points (4 in corners) normals */
CVect3D surface_normals[SURFACE_XMAX][SURFACE_ZMAX];
/* surface inner points (1 in the middle) normals */
CVect3D surface_midnormals[SURFACE_XMAX-1][SURFACE_ZMAX-1];

/* camera position */
CVect3D pos = CVect3D::zero;
/* camera look vector */
CVect3D lkt(1.0, 0.0, 0.0);
/* camera look-up vector */
CVect3D lku(0.0, 1.0, 0.0);

static int xDrag;
static int yDrag;
/* texture handles */
static unsigned int tex_trava;
static unsigned int tex_mraky;

int main( int argc, char **argv )
 {
	unsigned int i;

	srand( time( NULL ) );
	for( i = 0; i < sizeof(keys); i++ ) keys[i] = 0;
	tea_pos.y = generateSurface() * SURFACE_UP2;

	glutInit( &argc, argv );
	glutInitDisplayMode( GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH );
	glutInitWindowPosition( 0, 0 );
	glutInitWindowSize( WINDOW_WIDTH, WINDOW_HEIGHT );
	glutCreateWindow( WINDOW_TITLE );

	glutDisplayFunc( &guiDisplay );
	glutReshapeFunc( &guiReshape );
	glutKeyboardFunc( &guiKeyboard );
	glutKeyboardUpFunc( &guiKeyboardUp );
	glutMouseFunc( &guiMouse );
	glutMotionFunc( &guiMouseActive );
	glutPassiveMotionFunc ( & guiMousePassive );
	glutIdleFunc( &guiIdle );
	glutTimerFunc( MOTION_TIMER, &guiTimer, 0 );

	// initial state of light will be enabled
	guiKeyboard( 'l', 0, 0 );
	// using textures - set 0 and enable them
	guiKeyboard( '0', 0, 0 );
	guiKeyboard( 't', 0, 0 );
	
	// GLUT main loop
	glutMainLoop ();
	
	//deleting textures
	ZrusTexturu( &tex_trava );
	ZrusTexturu( &tex_mraky );
 
	return 0;
 }

void guiDisplay()
 {
	float dark = sun * CVect3D( 0.0, 1.0, 0.0 );
	GLboolean light_state;
	
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	
	/* processing lights */
	float lightPos[4];
	lightPos[0] = sun.x; lightPos[1] = sun.y; lightPos[2] = sun.z; lightPos[3] = 0.0; 
	glLightfv( GL_LIGHT0, GL_POSITION, lightPos );
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	glLoadIdentity();   // torch is in hand - camera position
	lightPos[0] = 0.0; lightPos[1] = 0.0; lightPos[2] = 0.0; lightPos[3] = 1.0; 
	glLightfv( GL_LIGHT1, GL_POSITION, lightPos );
	glPopMatrix();
	
	/* processing fog (color depends on darkness of skybox) */
	float fogColor[4];
	fogColor[0] = 0.8*dark; fogColor[1] = 0.8*dark; fogColor[2] = 0.8*dark; fogColor[3] = 0.0;
	glFogfv( GL_FOG_COLOR, fogColor );
	
	/* drawing surface */
	//glCullFace( GL_BACK );
	//glEnable( GL_CULL_FACE); 
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	glTranslatef( 0.0, 0.0, 0.0 );
	drawSurface();
	glPopMatrix();
	
	/* drawing skybox, manual computation of light */
	glGetBooleanv( GL_LIGHTING, &light_state );   // light is irelevant for skybox, we will restore it's state later
	glDisable( GL_LIGHTING );
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	glTranslatef( pos.x, pos.y, pos.z );   // skybox is always in same distance from camera
	//glutSolidCube( 1000.0 );
	drawSkybox();
	glPopMatrix();
	if( light_state ) glEnable( GL_LIGHTING );

	/* drawing moving teapot */
	if( wired ) glPolygonMode( GL_FRONT_AND_BACK, GL_LINE ); else glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
	glDisable( GL_TEXTURE_2D );
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	glTranslatef( tea_pos.x, tea_pos.y, tea_pos.z );
	glEnable( GL_COLOR_MATERIAL );
	glColorMaterial( GL_FRONT_AND_BACK, GL_DIFFUSE );
	glColor3f( 1.0, 0.1, 0.0 );
	glutSolidTeapot( 3.0 );
	glPopMatrix();
	
	/* drawing motion vectors */
	glGetBooleanv( GL_LIGHTING, &light_state );
	glDisable( GL_LIGHTING );
	glLineWidth( 1 );
	glDisable( GL_TEXTURE_2D );
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	glLoadIdentity();
	glTranslatef( 0.0, 0.0, -1.0 );
	glScalef( 0.1, 0.1, 0.1 );
	glLineWidth( 2 );
	glBegin( GL_LINES );
	 glColor3f( 0.0, 1.0, 1.0 );
	 glVertex3f( 0.0, 0.0, 0.0 );
	 glVertex3f( lkt.x, lkt.y, lkt.z );
	 glColor3f( 1.0, 0.0, 0.0);
	 glVertex3f( 0.0, 0.0, 0.0 );
	 glVertex3f( lku.x, lku.y, lku.z );
	glEnd();
	glPopMatrix();
	if( light_state ) glEnable( GL_LIGHTING );

	glutSwapBuffers();
	return;
 }

void guiReshape( int width, int height )
 {

	glViewport( 0, 0, width, height );

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity(); 
	gluPerspective( 90.0, 4.0/3.0, 0.1, 5000.0 );   // farest visible point in 5km

	glEnable( GL_DEPTH_TEST );
	glDepthFunc( GL_LEQUAL );
	glDepthRange( 0, 1 );

	return;
 } 

void guiIdle( )
 {
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	if( tea_attach )
	 {
		pos = tea_pos;
		pos.y += 5.0;   // little above teapot
	 }
	gluLookAt( pos.x, pos.y, pos.z, pos.x+lkt.x, pos.y+lkt.y, pos.z+lkt.z, lku.x, lku.y, lku.z );
	glutPostRedisplay();
	return;
 }

void guiTimer( int value )
 {
	CVect3D lkt_old;
	CVect3D lku_old;

	unsigned int key;
	for( key = 0; key < sizeof(keys); key++ )
	 {
		if( !keys[key] ) continue;
		switch( (unsigned char)key )
		 {
		  case 'w':
			pos += ~lkt * MOTION_STEP*keys[key];
			break;
		  case 's':
			pos -= ~lkt * MOTION_STEP*keys[key];
			break;
		  case 'a':
			pos -= ~(lkt ^ lku) * MOTION_STEP*keys[key];
			break;
		  case 'd':
			pos += ~(lkt ^ lku) * MOTION_STEP*keys[key];
			break;
		  case 'q':
			//lkt.rotate( MOTION_ANGLE*keys[key], lku );
			lkt.rotate( MOTION_ANGLE*keys[key], CVect3D(0,1,0) );
			lku.rotate( MOTION_ANGLE*keys[key], CVect3D(0,1,0) );
			break;		
		  case 'e':
			//lkt.rotate( -MOTION_ANGLE*keys[key], lku );
			lkt.rotate( -MOTION_ANGLE*keys[key], CVect3D(0,1,0) );
			lku.rotate( -MOTION_ANGLE*keys[key], CVect3D(0,1,0) );
			break;
		  case 'z':
			pos -= lku * MOTION_STEP*keys[key];
			break;
		  case 'x':
			pos += lku * MOTION_STEP*keys[key];
			break;
		  case 'r':
			lkt_old = lkt;
			lku_old = lku;
			lkt.rotate( MOTION_ANGLE*keys[key], ~(lkt ^ lku) );
			lku.rotate( MOTION_ANGLE*keys[key], ~(lkt ^ lku) );
			if( lku * CVect3D(0,1,0) < 0 )   // if we rotated over vertical axis limit, restore old values
			 {
				lkt = lkt_old;
				lku = lku_old;
			 }
			break;
		  case 'f':
			lkt_old = lkt;
			lku_old = lku;
			lkt.rotate( -MOTION_ANGLE*keys[key], ~(lkt ^ lku) );
			lku.rotate( -MOTION_ANGLE*keys[key], ~(lkt ^ lku) );
			if( lku * CVect3D(0,1,0) < 0 )
			 {
				lkt = lkt_old;
				lku = lku_old;
			 }
			break;
		  case 'c':   // tilt
			lku.rotate( -MOTION_ANGLE*keys[key], lkt );
			break;
		  case 'v':
			lku.rotate( MOTION_ANGLE*keys[key], lkt );
			break;
		 }
	 }

	/* rotation of Sun (light with infinite distance) */
	sun.rotate( SUN_VELOCI, CVect3D(0.0, 0.0, 1.0) );
	if( sun * CVect3D( 0.0, 1.0, 0.0 ) < 0 )   // for shorter night, sun will rotate 3 x faster
		sun.rotate( 2*SUN_VELOCI, CVect3D(0.0, 0.0, 1.0) );
	if( tea_motion )
	 {
		/* teapot is rotated around center of surface */
		tea_pos -= CVect3D( SURFACE_XMAX*SURFACE_GAP/2, 0.0, -SURFACE_ZMAX*SURFACE_GAP/2 );
		tea_pos.rotate( tea_veloci, CVect3D(0.0, 1.0, 0.0) );
		tea_pos += CVect3D( SURFACE_XMAX*SURFACE_GAP/2, 0.0, -SURFACE_ZMAX*SURFACE_GAP/2 );
		/* if camera is attached to teapot, we will rotate with it */
	 	if( tea_attach )
		 {
			//lkt.rotate( tea_veloci, lku );
			lkt.rotate( tea_veloci, CVect3D(0,1,0) );
			lku.rotate( tea_veloci, CVect3D(0,1,0) );
		 }
	 }

	glutTimerFunc( MOTION_TIMER, &guiTimer, 0 );
	return;
 }
 
void guiKeyboard( unsigned char key, int x, int y )
 {
	char tex_name[64];
	keys[key] = 1;   // in case of incrementation here, motion will be accelerated
	switch( key )
	 {
	  case '+': tea_veloci *= 1.1; break;
	  case '-': tea_veloci /= 1.1; break;
	  case 'p': tea_motion = !tea_motion; break;
	  case 'o': tea_attach = !tea_attach; break;
	  case 'n': normals = !normals;	break;
	  case 't':	textures = !textures; break;
	  case 'i':	wired = !wired; break;
	  // exact behaviour of displaying normals/textures/wired-model depends on each scene object
	  case 'g':
		tea_pos.y = generateSurface() * SURFACE_UP2;
		break;
	  case 'l':
		light = !light;
		if( light )
		 {
			glShadeModel( GL_SMOOTH );
			float lightColor[4];
			lightColor[0] = 0.1; lightColor[1] = 0.1; lightColor[2] = 0.1; lightColor[3] = 1.0;
			glLightModelfv( GL_LIGHT_MODEL_AMBIENT, lightColor );
			lightColor[0] = 0.9; lightColor[1] = 0.9; lightColor[2] = 0.9; lightColor[3] = 1.0;
			glLightfv( GL_LIGHT0, GL_DIFFUSE, lightColor );
			glEnable( GL_LIGHT0 );
			glEnable( GL_LIGHTING );	
		 } else {
			glShadeModel( GL_FLAT );
			glDisable( GL_LIGHTING );		
		 }
		break;
	  case 'm':
		glFogf( GL_FOG_DENSITY, 0.1 );
		glFogi( GL_FOG_MODE, GL_EXP );
		fog = !fog;
		if( fog ) glEnable( GL_FOG ); else glDisable( GL_FOG );
		break;
	  case 'b':
		torch = !torch;
		if( torch )
		 {
			float lightColor[4];
			lightColor[0] = 1.0; lightColor[1] = 1.0; lightColor[2] = 1.0; lightColor[3] = 1.0;
			glLightfv( GL_LIGHT1, GL_DIFFUSE, lightColor );
			glLightf( GL_LIGHT1, GL_SPOT_CUTOFF , 90.0 );
			glLightf( GL_LIGHT1, GL_SPOT_EXPONENT , 5.0 );
			glLightf( GL_LIGHT1, GL_CONSTANT_ATTENUATION , 1.0 );
			glLightf( GL_LIGHT1, GL_LINEAR_ATTENUATION , 0.0 );
			glLightf( GL_LIGHT1, GL_QUADRATIC_ATTENUATION , 0.01 );
			glEnable( GL_LIGHT1 );
		 } else {
			glDisable( GL_LIGHT1 );
		 }
		break;
	  case '0': case '1': case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9':
		// loading textures
		sprintf( tex_name, "tex\\trava-%c.bmp", key );
		if( NactiTexturu( tex_name, &tex_trava ) ) 
			fprintf( stdout, "Can't load texture \"%s\"\n", tex_name );
		sprintf( tex_name, "tex\\mraky-%c.bmp", key );
		if( NactiTexturu( tex_name, &tex_mraky ) ) 
			fprintf( stdout, "Can't load texture \"%s\"\n", tex_name );
		break;
	  case 27:   // ESC
		exit( 0 );
		break;
	 }
 }

void guiKeyboardUp( unsigned char key, int x, int y )
 {
	keys[key] = 0;
 }

void guiMouse( int button, int state, int x, int y )
 {
	xDrag = x;
	yDrag = y;
	return;
 }

void guiMouseActive( int x, int y )
 {
	CVect3D lkt_old;
	CVect3D lku_old;

	lkt.rotate( (xDrag-x)/MOTION_MOUSE, CVect3D(0.0, 1.0, 0.0) );
	lku.rotate( (xDrag-x)/MOTION_MOUSE, CVect3D(0.0, 1.0, 0.0) );
	lkt_old = lkt;
	lku_old = lku;
	lkt.rotate( (yDrag-y)/MOTION_MOUSE, ~(lkt_old ^ lku_old) );
	lku.rotate( (yDrag-y)/MOTION_MOUSE, ~(lkt_old ^ lku_old) );
	if( lku * CVect3D(0.0, 1.0, 0.0 ) < 0 )
	 {
		lkt = lkt_old;
		lku = lku_old;
	 }
	xDrag = x;
	yDrag = y;

	return;
 }

void guiMousePassive( int x, int y )
 {
	return;
 }

int generateSurface()   // returns level of highest point
 {
	unsigned int i, j;
	int z;
	char surfset[SURFACE_XMAX][SURFACE_ZMAX];
	int res;
	
	for( i = 0; i < SURFACE_XMAX; i++ )
		for( j = 0; j < SURFACE_ZMAX; j++ )
		 {
			surfset[i][j] = 0;
			//surface[i][j] = 0;
		 }
	/* generating corner points for surface squares */
	for( i = 0; i < SURFACE_XMAX; i++ )
		for( j = 0; j < SURFACE_ZMAX; j++ )
		 {
			if( surfset[i][j] ) continue;
			/* here we want odd numbers -...+SURFACE_TOP */
			/* even levels can be assigned to middle points only, not corners; 1 square makes up to 2 planes, not more */
			z = 2 * ( RAND0N( SURFACE_TOP ) - (int)(SURFACE_TOP/2) );
			generateSurfaceLift( i, j, z, surfset );
		 }
	/* generating middle points for surface squares */
	for( i = 0; i < SURFACE_XMAX-1; i++ )
		for( j = 0; j < SURFACE_ZMAX-1; j++ )
		 {
			char eq_ldlu = ( surface[i  ][j  ] == surface[i  ][j+1 ] );
			char eq_ldrd = ( surface[i  ][j  ] == surface[i+1][j  ] );
			char eq_ldru = ( surface[i  ][j  ] == surface[i+1][j+1] );
			char eq_lurd = ( surface[i  ][j+1] == surface[i+1][j  ] );
			char eq_luru = ( surface[i  ][j+1] == surface[i+1][j+1] );
			char eq_rdru = ( surface[i+1][j  ] == surface[i+1][j+1] );

			/* 2 or more rectangular level matches */ 
			if( eq_ldlu + eq_ldrd + eq_luru + eq_rdru  >=  2 )
			 {
				/* 3 on same level -> middle on same level */
				if     ( eq_ldlu && eq_ldrd ) surface_mids[i][j] = surface[i  ][j  ];
				else if( eq_ldlu && eq_luru ) surface_mids[i][j] = surface[i  ][j+1];
				else if( eq_ldrd && eq_rdru ) surface_mids[i][j] = surface[i+1][j  ];
				else if( eq_luru && eq_rdru ) surface_mids[i][j] = surface[i+1][j+1];
				/* now, there must be two down and two up in one plane (descendig or ascending) */
				/* -> middle on "inner" level, (2x+2x+(2x+2)+(2x+2)) / 4 = x+1 */
				else surface_mids[i][j] = ( surface[i][j] + surface[i][j+1] + surface[i+1][j] + surface[i+1][j+1] ) / 4;
 			 }
			/* 1 match is impossible - other two would be +/- 4 levels */
			/* 0 matches */
			else 
			 {
				/* middle on diagonal from left-down to right-up corner */
				if     ( eq_ldru ) surface_mids[i][j] = surface[i][j  ];
				/* if the condition above is false, other diagonal (left-up to right-down) exists */
				else if( eq_lurd ) surface_mids[i][j] = surface[i][j+1];
				/* here we should never come, if so, surface must be inconsistent (eg. 2 nearby corners have level diff > 2) */
				else surface_mids[i][j] = SURFACE_TOP;
			 }
		 }
	/* generating normals for corner points */
	for( i = 0; i < SURFACE_XMAX; i++ )
		for( j = 0; j < SURFACE_ZMAX; j++ )
		 {
			/* on border extrapolate with border surface levels */
			int levl = ( ( i > 0                ) ? surface[i-1][j  ] : surface[i][j] );
			int levr = ( ( i < SURFACE_XMAX - 1 ) ? surface[i+1][j  ] : surface[i][j] );
			int levd = ( ( j > 0                ) ? surface[i  ][j-1] : surface[i][j] );
			int levu = ( ( j < SURFACE_ZMAX - 1 ) ? surface[i  ][j+1] : surface[i][j] );
			/* levels of colliding middle points (diagonal path) */
			int levld = ( ( i > 0                && j > 0                ) ? surface_mids[i-1][j-1] : surface[i][j] );
			int levrd = ( ( i < SURFACE_XMAX - 1 && j > 0                ) ? surface_mids[i  ][j-1] : surface[i][j] );
			int levlu = ( ( i > 0                && j < SURFACE_ZMAX - 1 ) ? surface_mids[i-1][j  ] : surface[i][j] );
			int levru = ( ( i < SURFACE_XMAX - 1 && j < SURFACE_ZMAX - 1 ) ? surface_mids[i  ][j  ] : surface[i][j] );
			/* count horizontal and vertical diffs between nearby points and add rectangularly to current normal vector */
			float deltaH = SURFACE_GAP * 2;
			float deltaV = SURFACE_UP2 * (levr - levl);
			surface_normals[i][j] = CVect3D( -deltaV, deltaH, 0.0 );
			deltaV = SURFACE_UP2 * ( levd - levu );
			surface_normals[i][j] += CVect3D( 0.0, deltaH, -deltaV );
			/* adjust with colliding middle points (diagonal path), similar to generating their normals */
			deltaV = SURFACE_UP2 * ( levru - levld );
			surface_normals[i][j] += CVect3D( -deltaV, deltaH, -(-deltaV) );
			deltaV = SURFACE_UP2 * ( levrd - levlu );
			surface_normals[i][j] += CVect3D( -deltaV, deltaH, -deltaV );
			surface_normals[i][j].norm();
		 }
    /* generating normals for middle points */
	for( i = 0; i < SURFACE_XMAX-1; i++ )
		for( j = 0; j < SURFACE_ZMAX-1; j++ )
		 {
			float deltaH = SURFACE_GAP;
			float deltaV = SURFACE_UP2 * ( surface[i+1][j+1] - surface[i][j] );
			surface_midnormals[i][j] = CVect3D( -deltaV, deltaH, 0.0 );
			surface_midnormals[i][j] += CVect3D( 0.0, deltaH, -(-deltaV) );
			deltaV = SURFACE_UP2 * ( surface[i+1][j] - surface[i][j+1] );
			surface_midnormals[i][j] += CVect3D( -deltaV, deltaH, 0.0 );
			surface_midnormals[i][j] += CVect3D( 0.0, deltaH, -deltaV );
			surface_midnormals[i][j].norm();
		 }
	/* computing highest level really used */
	res = -SURFACE_TOP;
	for( i = 0; i < SURFACE_XMAX-1; i++ )
		for( j = 0; j < SURFACE_ZMAX-1; j++ )
			if( surface[i][j] > res ) res = surface[i][j];
    return res;
 }

void generateSurfaceLift( unsigned int i, unsigned int j, int z, char (*surfset)[SURFACE_ZMAX] )
 {
	surface[i][j] = z;
	surfset[i][j] = 1;
	if	   ( i > 0              && surface[i-1][j] < surface[i][j] - 2 )
		generateSurfaceLift( i-1, j, surface[i][j] - 2*RAND0N(1), surfset );
	else if( i > 0              && surface[i-1][j] > surface[i][j] + 2 )
		generateSurfaceLift( i-1, j, surface[i][j] + 2*RAND0N(1), surfset );
	if	   ( j > 0              && surface[i][j-1] < surface[i][j] - 2 )
		generateSurfaceLift( i, j-1, surface[i][j] - 2*RAND0N(1), surfset );
	else if( j > 0              && surface[i][j-1] > surface[i][j] + 2 )
		generateSurfaceLift( i, j-1, surface[i][j] + 2*RAND0N(1), surfset );
	if	   ( i < SURFACE_XMAX-1 && surface[i+1][j] < surface[i][j] - 2 )
		generateSurfaceLift( i+1, j, surface[i][j] - 2*RAND0N(1), surfset );
	else if( i < SURFACE_XMAX-1 && surface[i+1][j] > surface[i][j] + 2 )
		generateSurfaceLift( i+1, j, surface[i][j] + 2*RAND0N(1), surfset );
	if	   ( j < SURFACE_ZMAX-1 && surface[i][j+1] < surface[i][j] - 2 )
		generateSurfaceLift( i, j+1, surface[i][j] - 2*RAND0N(1), surfset );
	else if( j < SURFACE_ZMAX-1 && surface[i][j+1] > surface[i][j] + 2 )
		generateSurfaceLift( i, j+1, surface[i][j] + 2*RAND0N(1), surfset );
	return;
 }

void drawSurface()
 {
	int i, j;

	if( wired )
	 {
		// when wired-model is enabled, we will at first fill triangles with black (and Z buffer) and then display only visible lines (with clearing it's more illustrative)
		glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
		glColor3f( 0.0, 0.0, 0.0 );
		for( i = 0; i < SURFACE_XMAX-1; i++ )
			for( j = 0; j < SURFACE_ZMAX-1; j++ )
			 {
				glBegin( GL_TRIANGLE_FAN );			
				 glVertex3f( i*SURFACE_GAP + SURFACE_GAP/2, surface_mids[i][j]*SURFACE_UP2, -(j*SURFACE_GAP + SURFACE_GAP/2) );
				 glVertex3f( (i  )*SURFACE_GAP, surface[i  ][j  ]*SURFACE_UP2, -((j  )*SURFACE_GAP) );
				 glVertex3f( (i+1)*SURFACE_GAP, surface[i+1][j  ]*SURFACE_UP2, -((j  )*SURFACE_GAP) );
				 glVertex3f( (i+1)*SURFACE_GAP, surface[i+1][j+1]*SURFACE_UP2, -((j+1)*SURFACE_GAP) );
				 glVertex3f( (i  )*SURFACE_GAP, surface[i  ][j+1]*SURFACE_UP2, -((j+1)*SURFACE_GAP) );
				 glVertex3f( (i  )*SURFACE_GAP, surface[i  ][j  ]*SURFACE_UP2, -((j  )*SURFACE_GAP) );
				glEnd();
			 }
		glPolygonMode( GL_FRONT_AND_BACK, GL_LINE ); 
		glLineWidth( 3 );   // not all pixels are processed accurately due to float imprecision in (L)equal test when Z buffer is filled with black triangles
	 } else glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
	if( textures )
	 {
		glBindTexture( GL_TEXTURE_2D, tex_trava );
		glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
		glEnable( GL_TEXTURE_2D );
	 } else glDisable( GL_TEXTURE_2D );

	glEnable( GL_COLOR_MATERIAL );
	glColorMaterial( GL_FRONT_AND_BACK, GL_DIFFUSE );
	glColor3f( 0.3, 1.0, 0.1 );

	for( i = 0; i < SURFACE_XMAX-1; i++ )
		for( j = 0; j < SURFACE_ZMAX-1; j++ )
		 {
			glBegin( GL_TRIANGLE_FAN );			
			 glNormal3f( surface_midnormals[i][j].x, surface_normals[i][j].y, surface_normals[i][j].z );
			 glTexCoord2f( 0.5, 0.5 );
			 glVertex3f( i*SURFACE_GAP + SURFACE_GAP/2, surface_mids[i][j]*SURFACE_UP2, -(j*SURFACE_GAP + SURFACE_GAP/2) );
			 glNormal3f( surface_normals[i  ][j  ].x, surface_normals[i  ][j  ].y, surface_normals[i  ][j  ].z );
			 glTexCoord2f( 0.0, 0.0 );
			 glVertex3f( (i  )*SURFACE_GAP, surface[i  ][j  ]*SURFACE_UP2, -((j  )*SURFACE_GAP) );
			 glNormal3f( surface_normals[i+1][j  ].x, surface_normals[i+1][j  ].y, surface_normals[i+1][j  ].z );
			 glTexCoord2f( 1.0, 0.0 );
			 glVertex3f( (i+1)*SURFACE_GAP, surface[i+1][j  ]*SURFACE_UP2, -((j  )*SURFACE_GAP) );
			 glNormal3f( surface_normals[i+1][j+1].x, surface_normals[i+1][j+1].y, surface_normals[i+1][j+1].z );
			 glTexCoord2f( 1.0, 1.0 );
			 glVertex3f( (i+1)*SURFACE_GAP, surface[i+1][j+1]*SURFACE_UP2, -((j+1)*SURFACE_GAP) );
			 glNormal3f( surface_normals[i  ][j+1].x, surface_normals[i  ][j+1].y, surface_normals[i  ][j+1].z );
			 glTexCoord2f( 0.0, 1.0 );
			 glVertex3f( (i  )*SURFACE_GAP, surface[i  ][j+1]*SURFACE_UP2, -((j+1)*SURFACE_GAP) );
			 glNormal3f( surface_normals[i  ][j  ].x, surface_normals[i  ][j  ].y, surface_normals[i  ][j  ].z );
			 glTexCoord2f( 0.0, 0.0 );
			 glVertex3f( (i  )*SURFACE_GAP, surface[i  ][j  ]*SURFACE_UP2, -((j  )*SURFACE_GAP) );
			glEnd();
		 }
	/* trees */
	for( i = 0; i < SURFACE_XMAX-1; i++ ) for( j = 0; j < SURFACE_ZMAX-1; j++ )	
	 {
		if( surface[i][j] == surface[i  ][j+1] &&
			surface[i][j] == surface[i+1][j  ] &&
			surface[i][j] == surface[i+1][j+1] )
		 {
			glMatrixMode( GL_MODELVIEW );
			glPushMatrix();
			glTranslatef( i*SURFACE_GAP + SURFACE_GAP/2, surface_mids[i][j]*SURFACE_UP2, -(j*SURFACE_GAP + SURFACE_GAP/2) );
			drawTree( 1.0 + ((i*j) % 7 - 2)/4.0 );
			glPopMatrix();
		 }
	 }

	if( normals )
	 {
		glLineWidth( 3 ); 
		glDisable( GL_TEXTURE_2D );
		for( i = 0; i < SURFACE_XMAX; i++ ) for( j = 0; j < SURFACE_ZMAX; j++ )
		 {
			glColor3f( 1.0, 0.2, 0.0 );
			glBegin( GL_LINES );
			 glVertex3f( (i  )*SURFACE_GAP, surface[i  ][j  ]*SURFACE_UP2, -((j  )*SURFACE_GAP) );
			 glVertex3f( (i  )*SURFACE_GAP + surface_normals[i][j].x,
						 surface[i  ][j  ]*SURFACE_UP2 + surface_normals[i][j].y,
						 -((j  )*SURFACE_GAP) + surface_normals[i][j].z
					   );
			glEnd();
		 }
		for( i = 0; i < SURFACE_XMAX-1; i++ ) for( j = 0; j < SURFACE_ZMAX-1; j++ )
		 {
			glColor3f( 1.0, 0.8, 0.0 );
			glBegin( GL_LINES );
			 glVertex3f( i*SURFACE_GAP + SURFACE_GAP/2, surface_mids[i][j]*SURFACE_UP2, -(j*SURFACE_GAP + SURFACE_GAP/2) );
			 glVertex3f( i*SURFACE_GAP + SURFACE_GAP/2 + surface_midnormals[i][j].x,
						 surface_mids[i][j]*SURFACE_UP2 + surface_midnormals[i][j].y,
						 -(j*SURFACE_GAP + SURFACE_GAP/2) + surface_midnormals[i][j].z
					   );
			glEnd();
		 }
	 }
	return;
 }

void drawTree( float treesize )
 {
	if( wired ) glPolygonMode( GL_FRONT_AND_BACK, GL_LINE ); else glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
	glLineWidth( 2 );
	glDisable( GL_TEXTURE_2D );
	
	/*glEnable( GL_RESCALE_NORMALS );*/ glEnable( GL_NORMALIZE );
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();	
	glScalef( treesize, treesize, treesize );
	glRotatef( -90.0, 1.0, 0.0, 0.0 );
	
	glEnable( GL_COLOR_MATERIAL );
	glColorMaterial( GL_FRONT_AND_BACK, GL_DIFFUSE );
	glColor3f( 0.9, 0.4, 0.1 );
	glutSolidCone( 0.25, 3.0, 5, 2 );

	glColorMaterial( GL_FRONT_AND_BACK, GL_DIFFUSE );
	glColor3f( 0.2, 0.8, 0.6 );
	glTranslatef( 0.0, 0.0, 0.9 );
	glutSolidCone( 0.8, 1.5, 9, 2 );
	glTranslatef( 0.0, 0.0, 0.8 );
	glutSolidCone( 0.6, 1.25, 9, 2 );
	glTranslatef( 0.0, 0.0, 0.7 );
	glutSolidCone( 0.4, 1.0, 9, 2 );

	glPopMatrix();
	/*glDisable( GL_RESCALE_NORMALS );*/ glDisable( GL_NORMALIZE );
	return;
 }

void drawSkybox()
 {
	float dark = sun * CVect3D( 0.0, 1.0, 0.0 );
 	float blendColor[4];

	if( dark > 0 )   // color of skybox depends on daytime, at night there will be darkness
	 {
		glColor3f( 0.1*dark, 0.5*dark, 0.7*dark );
		blendColor[0] = dark; blendColor[1] = dark; blendColor[2] = dark; blendColor[3] = 0.0;
	 } else {
		glColor3f( 0.0, 0.0, 0.0 );
		blendColor[0] = 0.0; blendColor[1] = 0.0; blendColor[2] = 0.0; blendColor[3] = 0.0;
	 }
	glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
	if( textures )
	 {
		glBindTexture( GL_TEXTURE_2D, tex_mraky );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );		
		glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_BLEND );
		glTexEnvfv( GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, blendColor );
		glEnable( GL_TEXTURE_2D );
	 } else glDisable( GL_TEXTURE_2D );
/*	
	glBegin( GL_QUADS );
	 // upper plane
	 glTexCoord2f(  0.0,  0.0 ); glVertex3f( -1000.0,  100.0,  1000.0 );
	 glTexCoord2f( 10.0,  0.0 ); glVertex3f(  1000.0,  100.0,  1000.0 );
	 glTexCoord2f( 10.0, 10.0 ); glVertex3f(  1000.0,  100.0, -1000.0 );
	 glTexCoord2f(  0.0, 10.0 ); glVertex3f( -1000.0,  100.0, -1000.0 );
	glEnd();
*/
	GLUquadricObj *qobj = gluNewQuadric();

	gluQuadricDrawStyle(qobj, GLU_FILL); /* smooth shaded */
	gluQuadricNormals(qobj, GLU_NONE);
	gluQuadricTexture(qobj, GLU_TRUE);
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	glRotatef( -90.0, 1.0, 0.0, 0.0 );
	gluSphere(qobj, 1000.0, 12, 9);
	glPopMatrix();
	return;
 }

AUX_RGBImageRec* LoadBMP(char *filename)
{
	FILE *File=NULL;

	if (!filename) return NULL;
	File=fopen(filename,"r");
	if (File)
	{
		fclose(File);
		return auxDIBImageLoad(filename);
	}
	return NULL;
}

int NactiTexturu (char *filename, unsigned int* handle)
{
	int Status=1;

	AUX_RGBImageRec *TextureImage=NULL;

	// Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
	if ((TextureImage=LoadBMP(filename)))
	{
		Status=0;
		glGenTextures(1, handle);

		// Create Linear Filtered Texture
		glBindTexture(GL_TEXTURE_2D, *handle);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D, 0, 3, TextureImage->sizeX, TextureImage->sizeY, 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage->data);
	}

	if (TextureImage)
	{
		if (TextureImage->data) free(TextureImage->data);
		free(TextureImage);
	}

	return Status;
}

void ZrusTexturu (unsigned int* handle)
{
	glDeleteTextures (1, handle);
}
